In progress. Manage a directory tree containing plain-text notes (with [some
line drawings](http://akkartik.name/lines.html)). Notes/files connect to each
other using [symmetric cross-links](https://merveilles.town/@akkartik/108766067153506592)
which are saved outside the plain-text in a corresponding .json file.

Some features:
- Multiple views into an independent repo of plain text notes (with [some line
  drawings](http://akkartik.name/lines.html)).
- Notes can have arbitrary named relationships (links) to each other,
  resulting in a graph.
- The primary visual metaphor is an infinite 2D surface of fixed-width,
  non-overlapping, notes that can be operated on with an extensible set of
  commands.
- Commands usually (but not always) create new columns of notes. Each column
  is an independent visualization of a subset of notes.
- Find on surface.
- Search all notes on disk in reverse-chronological order.
- Notes can contain hyperlinks in [[wikiwords]] to other notes. Hyperlinks
  can't contain spaces. Clicking on a hyperlink opens the target in a new
  column.
- A few other things. Explore the command menu and palette along the top.

Some videos to help visualize these bullets (albeit with outdated UI):
  * https://archive.org/details/akkartik-pensieve-2022-07-27 (4.5 minutes)
  * https://merveilles.town/@akkartik/108766067153506592 (30 seconds)
  * https://archive.org/details/akkartik-pensieve-2022-08-29 (3 minutes)

pensieve.love is a compatible fork of [lines.love](http://akkartik.name/lines.html),
an editor for plain text where you can also seamlessly insert line drawings.
Designed above all to be easy to modify and give you early warning if your
modifications break something.

## Getting started

Install [LÖVE](https://love2d.org). It's just a 5MB download, open-source and
extremely well-behaved. I'll assume below that you can invoke it using the
`love` command, but that might vary depending on your OS.

The first time you run pensieve.love, do so from a terminal and pass in the
directory where you want to store your notes.

(The repo contains a 'data.sample/' directory for trying out with.)

From here on out, restarting pensieve.love will open the same directory.
Run again from a terminal if you ever want to switch to a new directory.

## Keyboard shortcuts

While editing text:
* `ctrl+f` to find patterns within a file
* `ctrl+c` to copy, `ctrl+x` to cut, `ctrl+v` to paste
* `ctrl+z` to undo, `ctrl+y` to redo
* `ctrl+=` to zoom in, `ctrl+-` to zoom out, `ctrl+0` to reset zoom
* `alt+right`/`alt+left` to jump to the next/previous word, respectively
* mouse drag or `shift` + movement to select text, `ctrl+a` to select all
* `ctrl+w` to modify the sources

For shortcuts while editing drawings, consult the online help. Either:
* hover on a drawing and hit `ctrl+h`, or
* click on a drawing to start a stroke and then press and hold `h` to see your
  options at any point during a stroke.

pensieve.love has been exclusively tested so far with a US keyboard layout. If
you use a different layout, please let me know if things worked, or if you
found anything amiss: http://akkartik.name/contact

## Known issues

* No support yet for Unicode graphemes spanning multiple codepoints.

* No support yet for right-to-left languages.

* Editing a note larger than 50KB grows sluggish (particularly during
  undo/redo) even when maximized.

* If you kill the process, say by force-quitting because things things get
  sluggish, you can lose data.

* Searching is slow. There's no index yet. Searching happens in the same
  thread as UI updates, so UI may go unresponsive with really large notes. I
  haven't observed this in practice.

* Pressing keys while editing will try to always keep the text cursor on
  screen. This can have some strange implications:

    * A long series of drawings will get silently skipped when you hit
      page-down, until a line of text can be showed on screen.
    * If there's no line of text at the top of the file, you may not be able
      to scroll back up to the top with page-up.

  So far this app isn't really designed for drawing-heavy files. For now I'm
  targeting mostly-text files with a few drawings mixed in.

* No clipping yet for drawings. In particular, circles/squares/rectangles and
  point labels can overflow a drawing.

* Touchpads can drag the mouse pointer using a light touch or a heavy click.
  On Linux, drags using the light touch get interrupted when a key is pressed.
  You'll have to press down to drag.

* Can't scroll while selecting text with mouse.

* No scrollbars yet. That stuff is hard.

## Mirrors and Forks

This repo is a fork of [lines.love](http://akkartik.name/lines.html), an
editor for plain text where you can also seamlessly insert line drawings.
Updates to it can be downloaded from:

* https://git.sr.ht/~akkartik/pensieve.love
* https://tildegit.org/akkartik/pensieve.love
* https://git.merveilles.town/akkartik/pensieve.love
* https://nest.pijul.com/akkartik/pensieve.love (using the Pijul version control system)

Further forks are encouraged. If you show me your fork, I'll link to it here.

## Associated tools

* https://git.sr.ht/~akkartik/capture.love provides a blank-slate UI into the
  same notes directory as pensieve.love.
* https://codeberg.org/akkartik/lines2md exports lines.love files to Markdown
  and (non-editable) SVG.
* https://git.sr.ht/~akkartik/lines2html.love exports lines.love files to html
  and inline SVG.

## Feedback

[Most appreciated.](http://akkartik.name/contact)

In particular, pensieve.love has been exclusively tested so far with a US
keyboard layout. If you use a different layout, please let me know if things
worked, or if you found anything amiss.
